@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2000,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2000-2009 Oracle.  All rights reserved.
  *
- * $Id: RecordNumberBinding.java,v 12.5 2007/05/04 00:28:24 mark Exp $
+ * $Id$
  */
 
 package com.sleepycat.bind;
@@ -29,9 +29,9 @@ public class RecordNumberBinding implements EntryBinding {
     }
 
     // javadoc is inherited
-    public Object entryToObject(DatabaseEntry entry) {
+    public Long entryToObject(DatabaseEntry entry) {
 
-        return new Long(entryToRecordNumber(entry));
+        return Long.valueOf(entryToRecordNumber(entry));
     }
 
     // javadoc is inherited
