@@ -1,9 +1,9 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 1997,2007 Oracle.  All rights reserved.
+ * Copyright (c) 1997-2009 Oracle.  All rights reserved.
  *
- * $Id: os_dir.c,v 12.12 2007/05/17 15:15:46 bostic Exp $
+ * $Id$
  */
 
 #include "db_config.h"
@@ -15,59 +15,72 @@
  * __os_dirlist --
  *	Return a list of the files in a directory.
  *
- * PUBLIC: int __os_dirlist __P((DB_ENV *, const char *, char ***, int *));
+ * PUBLIC: int __os_dirlist __P((ENV *, const char *, int, char ***, int *));
  */
 int
-__os_dirlist(dbenv, dir, namesp, cntp)
-	DB_ENV *dbenv;
+__os_dirlist(env, dir, returndir, namesp, cntp)
+	ENV *env;
 	const char *dir;
+	int returndir, *cntp;
 	char ***namesp;
-	int *cntp;
 {
+	DB_ENV *dbenv;
 	struct dirent *dp;
 	DIR *dirp;
 	struct stat sb;
 	int arraysz, cnt, ret;
 	char **names, buf[DB_MAXPATHLEN];
 
+	*namesp = NULL;
+	*cntp = 0;
+
+	dbenv = env == NULL ? NULL : env->dbenv;
+
 	if (dbenv != NULL &&
 	    FLD_ISSET(dbenv->verbose, DB_VERB_FILEOPS | DB_VERB_FILEOPS_ALL))
-		__db_msg(dbenv, "fileops: directory list %s", dir);
+		__db_msg(env, "fileops: directory list %s", dir);
 
 	if (DB_GLOBAL(j_dirlist) != NULL)
 		return (DB_GLOBAL(j_dirlist)(dir, namesp, cntp));
 
-#ifdef HAVE_VXWORKS
-	if ((dirp = opendir((char *)dir)) == NULL)
-#else
-	if ((dirp = opendir(dir)) == NULL)
-#endif
+	if ((dirp = opendir(CHAR_STAR_CAST dir)) == NULL)
 		return (__os_get_errno());
 	names = NULL;
 	for (arraysz = cnt = 0; (dp = readdir(dirp)) != NULL;) {
 		snprintf(buf, sizeof(buf), "%s/%s", dir, dp->d_name);
 
-		/*
-		 * We're only interested in regular files, skip the rest.
-		 * Other parts of Berkeley DB depend on this, for example,
-		 * the db_hotbackup utility doesn't want to try and copy
-		 * directories.
-		 */
 		RETRY_CHK(stat(buf, &sb), ret);
 		if (ret != 0) {
 			ret = __os_posix_err(ret);
 			goto err;
 		}
-		if (!S_ISREG(sb.st_mode))
-			continue;
+
+		/*
+		 * We return regular files, and optionally return directories
+		 * (except for dot and dot-dot).
+		 *
+		 * Shared memory files are of a different type on QNX, and we
+		 * return those as well.
+		 */
+#ifdef HAVE_QNX
+		if (!S_ISREG(sb.st_mode) && !S_TYPEISSHM(&sb)) {
+#else
+		if (!S_ISREG(sb.st_mode)) {
+#endif
+			if (!returndir || !S_ISDIR(sb.st_mode))
+				continue;
+			if (dp->d_name[0] == '.' && (dp->d_name[1] == '\0' ||
+			    (dp->d_name[1] == '.' && dp->d_name[2] == '\0')))
+				continue;
+		}
 
 		if (cnt >= arraysz) {
 			arraysz += 100;
-			if ((ret = __os_realloc(dbenv,
+			if ((ret = __os_realloc(env,
 			    (u_int)arraysz * sizeof(names[0]), &names)) != 0)
 				goto err;
 		}
-		if ((ret = __os_strdup(dbenv, dp->d_name, &names[cnt])) != 0)
+		if ((ret = __os_strdup(env, dp->d_name, &names[cnt])) != 0)
 			goto err;
 		cnt++;
 	}
@@ -78,7 +91,7 @@ __os_dirlist(dbenv, dir, namesp, cntp)
 	return (0);
 
 err:	if (names != NULL)
-		__os_dirfree(dbenv, names, cnt);
+		__os_dirfree(env, names, cnt);
 	if (dirp != NULL)
 		(void)closedir(dirp);
 	return (ret);
@@ -88,11 +101,11 @@ err:	if (names != NULL)
  * __os_dirfree --
  *	Free the list of files.
  *
- * PUBLIC: void __os_dirfree __P((DB_ENV *, char **, int));
+ * PUBLIC: void __os_dirfree __P((ENV *, char **, int));
  */
 void
-__os_dirfree(dbenv, names, cnt)
-	DB_ENV *dbenv;
+__os_dirfree(env, names, cnt)
+	ENV *env;
 	char **names;
 	int cnt;
 {
@@ -100,7 +113,7 @@ __os_dirfree(dbenv, names, cnt)
 		DB_GLOBAL(j_dirfree)(names, cnt);
 	else {
 		while (cnt > 0)
-			__os_free(dbenv, names[--cnt]);
-		__os_free(dbenv, names);
+			__os_free(env, names[--cnt]);
+		__os_free(env, names);
 	}
 }
