@@ -1,9 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2004-2006
-#	Oracle Corporation.  All rights reserved.
+# Copyright (c) 2004-2009 Oracle.  All rights reserved.
 #
-# $Id: rep020.tcl,v 12.10 2006/08/24 14:46:37 bostic Exp $
+# $Id$
 #
 # TEST  rep020
 # TEST	Replication elections - test election generation numbers.
@@ -11,6 +10,8 @@
 
 proc rep020 { method args } {
 	global rand_init
+	global databases_in_memory
+	global repfiles_in_memory
 
 	source ./include.tcl
 	if { $is_windows9x_test == 1 } {
@@ -32,8 +33,25 @@ proc rep020 { method args } {
 	error_check_good set_random_seed [berkdb srand $rand_init] 0
 	set nclients 5
 	set logsets [create_logsets [expr $nclients + 1]]
+
+	# Set up for on-disk or in-memory databases.
+	set msg "using on-disk databases"
+	if { $databases_in_memory } {
+		set msg "using named in-memory databases"
+		if { [is_queueext $method] } { 
+			puts -nonewline "Skipping rep$tnum for method "
+			puts "$method with named in-memory databases"
+			return
+		}
+	}
+
+	set msg2 "and on-disk replication files"
+	if { $repfiles_in_memory } {
+		set msg2 "and in-memory replication files"
+	}
+
 	foreach l $logsets {
-		puts "Rep$tnum ($method): Election generation test."
+		puts "Rep$tnum ($method): Election generation test $msg $msg2."
 		puts "Rep$tnum: Master logs are [lindex $l 0]"
 		for { set i 0 } { $i < $nclients } { incr i } {
 			puts "Rep$tnum: Client $i logs are\
@@ -46,7 +64,22 @@ proc rep020 { method args } {
 proc rep020_sub { method nclients tnum logset largs } {
 	source ./include.tcl
 	global errorInfo
+	global databases_in_memory
 	global mixed_mode_logging
+	global repfiles_in_memory
+	global rep_verbose
+	global verbose_type
+
+	set verbargs ""
+	if { $rep_verbose == 1 } {
+		set verbargs " -verbose {$verbose_type on} "
+	}
+
+	set repmemargs ""
+	if { $repfiles_in_memory } {
+		set repmemargs "-rep_inmem_files "
+	}
+
 	env_cleanup $testdir
 
 	set qdir $testdir/MSGQUEUEDIR
@@ -66,30 +99,25 @@ proc rep020_sub { method nclients tnum logset largs } {
 		set c_txnargs($i) [adjust_txnargs $c_logtype($i)]
 	}
 
-# To debug elections, the lines to uncomment are below the
-# error checking portion of this test.  This is needed in order
-# for the error messages to come back in errorInfo and for
-# that portion of the test to pass.
 	# Open a master.
 	set envlist {}
 	repladd 1
-	set env_cmd(M) "berkdb_env -create -log_max 1000000 \
+	set env_cmd(M) "berkdb_env_noerr -create -log_max 1000000 $verbargs \
+	    -event rep_event $repmemargs \
 	    -home $masterdir $m_txnargs $m_logargs -rep_master \
 	    -errpfx MASTER -rep_transport \[list 1 replsend\]"
 	set masterenv [eval $env_cmd(M)]
-	error_check_good master_env [is_valid_env $masterenv] TRUE
 	lappend envlist "$masterenv 1"
 
 	# Open the clients.
 	for { set i 0 } { $i < $nclients } { incr i } {
 		set envid [expr $i + 2]
 		repladd $envid
-		set env_cmd($i) "berkdb_env_noerr -create \
-		    -home $clientdir($i) $c_txnargs($i) $c_logargs($i) \
+		set env_cmd($i) "berkdb_env_noerr -create -event rep_event \
+		    $verbargs -home $clientdir($i) $repmemargs \
+		    $c_txnargs($i) $c_logargs($i) \
 		    -rep_client -rep_transport \[list $envid replsend\]"
 		set clientenv($i) [eval $env_cmd($i)]
-		error_check_good \
-		    client_env($i) [is_valid_env $clientenv($i)] TRUE
 		lappend envlist "$clientenv($i) $envid"
 	}
 
@@ -97,26 +125,37 @@ proc rep020_sub { method nclients tnum logset largs } {
 	process_msgs $envlist
 	puts "\tRep$tnum.a: Running rep_test in replicated env."
 	set niter 10
-	eval rep_test $method $masterenv NULL $niter 0 0 0 0 $largs
+	eval rep_test $method $masterenv NULL $niter 0 0 0 $largs
 	process_msgs $envlist
+
+	# Check that databases are in-memory or on-disk as expected.
+	if { $databases_in_memory } {
+		set dbname { "" "test.db" }
+	} else { 
+		set dbname "test.db"
+	} 
+	check_db_location $masterenv
+	for { set i 0 } { $i < $nclients } { incr i } { 
+		check_db_location $clientenv($i)
+	}
+
+	# Close master.
 	error_check_good masterenv_close [$masterenv close] 0
 	set envlist [lreplace $envlist 0 0]
 
-# To debug elections, uncomment the lines below to turn on verbose
-# and set the errfile.  Also edit reputils.tcl
-# in proc start_election and swap the 2 commented lines with
-# their counterpart.
 	foreach pair $envlist {
 		set i [expr [lindex $pair 1] - 2]
 		replclear [expr $i + 2]
 		set err_cmd($i) "none"
 		set pri($i) 10
 		set crash($i) 0
-#		error_check_good pfx [$clientenv($i) errpfx CLIENT$i] 0
-#		error_check_good verb [$clientenv($i) verbose rep on] 0
-#		$clientenv($i) errfile /dev/stderr
-#		set env_cmd($i) [concat $env_cmd($i) \
-#		    "-errpfx CLIENT$i -verbose {rep on} -errfile /dev/stderr"]
+		if { $rep_verbose == 1 } {
+			$clientenv($i) errpfx CLIENT$i
+			$clientenv($i) verbose $verbose_type on
+			$clientenv($i) errfile /dev/stderr
+			set env_cmd($i) [concat $env_cmd($i) \
+			    "-errpfx CLIENT$i -errfile /dev/stderr"]
+		}
 	}
 
 	set msg "Rep$tnum.b"
@@ -130,8 +169,8 @@ proc rep020_sub { method nclients tnum logset largs } {
 		set winner [berkdb random_int 0 [expr $nclients - 1]]
 		setpriority pri $nclients $winner
 		set elector [berkdb random_int 0 [expr $nclients - 1]]
-		run_election env_cmd envlist err_cmd pri crash\
-		    $qdir $msg $elector $nsites $nvotes $nclients $winner 1
+		run_election env_cmd envlist err_cmd pri crash $qdir \
+		    $msg $elector $nsites $nvotes $nclients $winner 1 $dbname
 	}
 	process_msgs $envlist
 
@@ -162,8 +201,8 @@ proc rep020_sub { method nclients tnum logset largs } {
 		set winner [berkdb random_int 0 [expr $nclients - 1]]
 		setpriority pri $nclients $winner
 		set elector [berkdb random_int 0 [expr $nclients - 1]]
-		run_election env_cmd envlist err_cmd pri crash\
-		    $qdir $msg $elector $nsites $nvotes $nclients $winner 1
+		run_election env_cmd envlist err_cmd pri crash $qdir \
+		    $msg $elector $nsites $nvotes $nclients $winner 1 $dbname
 	}
 	process_msgs $envlist
 	#
@@ -200,8 +239,8 @@ proc rep020_sub { method nclients tnum logset largs } {
 	set nclients $orig_nclients
 	set elector [expr $nclients - 1]
 	setpriority pri $nclients $winner
-	run_election env_cmd envlist err_cmd pri crash\
-	    $qdir $msg $elector $nsites $nvotes $nclients $winner 0
+	run_election env_cmd envlist err_cmd pri crash $qdir \
+	    $msg $elector $nsites $nvotes $nclients $winner 0 $dbname
 
 	set newegen($i) \
 	    [stat_field $clientenv($i) rep_stat "Election generation number"]
@@ -217,23 +256,27 @@ proc rep020_sub { method nclients tnum logset largs } {
 	# since we can't recover with in-memory logs.
 	if { $mixed_mode_logging == 0 } {
 		set msg "Rep$tnum.e"
-		puts "\t$msg: Election generation is not changed in recovery."
+	puts "\t$msg: Election generation set as expected after recovery."
 		# Note all client egens.  Close, recover, process messages,
 		# and check that egens are unchanged.
 		set big_e [big_endian]
 		foreach pair $envlist {
 			set i [expr [lindex $pair 1] - 2]
-			set fid [open $clientdir($i)/__db.rep.egen r]
-			fconfigure $fid -translation binary
-			set data [read $fid 4]
-			if { $big_e } {
-				binary scan $data I egen($i)
-			} else {
-				binary scan $data i egen($i)
-			}
-			binary scan $data c val
-			close $fid
 			set clientenv($i) [lindex $pair 0]
+			# Can only get egen file if repfiles on-disk.
+			if { $repfiles_in_memory == 0 } {
+				set fid [open $clientdir($i)/__db.rep.egen r]
+				fconfigure $fid -translation binary
+				set data [read $fid 4]
+				if { $big_e } {
+					binary scan $data I egen($i)
+				} else {
+					binary scan $data i egen($i)
+				}
+				binary scan $data c val
+				close $fid
+			}
+			$clientenv($i) log_flush
 			error_check_good \
 			    clientenv_close($i) [$clientenv($i) close] 0
 			set clientenv($i) [eval $env_cmd($i) -recover]
@@ -245,23 +288,36 @@ proc rep020_sub { method nclients tnum logset largs } {
 			set i [expr [lindex $pair 1] - 2]
 			set newegen($i) [stat_field $clientenv($i) \
 			    rep_stat "Election generation number"]
-			error_check_good egen_recovery $egen($i) $newegen($i)
+			if { $repfiles_in_memory == 0 } {
+				error_check_good egen_recovery $egen($i) \
+				    $newegen($i)
+			} else {
+				# For rep in-memory, egen expected to start
+				# over at 1 after close/reopen environment.
+				error_check_good egen_recovery $newegen($i) 1
+			}
 		}
 
 		# Run an election.  Now the egens should go forward.
 		set winner [berkdb random_int 0 [expr $nclients - 1]]
 		setpriority pri $nclients $winner
 		set elector [berkdb random_int 0 [expr $nclients - 1]]
-		run_election env_cmd envlist err_cmd pri crash \
-		    $qdir $msg $elector $nsites $nvotes $nclients $winner 1
+		run_election env_cmd envlist err_cmd pri crash $qdir \
+		    $msg $elector $nsites $nvotes $nclients $winner 1 $dbname
 
 		foreach pair $envlist {
 			set i [expr [lindex $pair 1] - 2]
 			set clientenv($i) [lindex $pair 0]
 			set newegen($i) [stat_field $clientenv($i) \
 			    rep_stat "Election generation number"]
-			error_check_good \
-			    egen_forward [expr $newegen($i) > $egen($i)] 1
+			if { $repfiles_in_memory == 0 } {
+				error_check_good egen_forward \
+				    [expr $newegen($i) > $egen($i)] 1
+			} else {
+				# For rep in-memory, egen expected to
+				# increment to 2 after election.
+				error_check_good egen_recovery $newegen($i) 2
+			}
 		}
 	}
 
