@@ -1,25 +1,27 @@
 /*-
  * See the file LICENSE for redistribution information.
  *
- * Copyright (c) 2002,2007 Oracle.  All rights reserved.
+ * Copyright (c) 2002-2009 Oracle.  All rights reserved.
  *
- * $Id: SecondaryDeadlockTest.java,v 12.7 2007/05/04 00:28:29 mark Exp $
+ * $Id$
  */
 
 package com.sleepycat.collections.test;
 
-import com.sleepycat.db.Database;
-import com.sleepycat.db.DeadlockException;
-import com.sleepycat.db.Environment;
-import com.sleepycat.db.TransactionConfig;
-import com.sleepycat.collections.StoredSortedMap;
-import com.sleepycat.collections.TransactionRunner;
-import com.sleepycat.collections.TransactionWorker;
-import com.sleepycat.util.ExceptionUnwrapper;
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
 
+import com.sleepycat.collections.StoredSortedMap;
+import com.sleepycat.collections.TransactionRunner;
+import com.sleepycat.collections.TransactionWorker;
+import com.sleepycat.db.Database;
+import com.sleepycat.db.Environment;
+import com.sleepycat.db.DeadlockException;
+import com.sleepycat.db.TransactionConfig;
+import com.sleepycat.util.ExceptionUnwrapper;
+import com.sleepycat.util.test.TestEnv;
+
 /**
  * Tests whether secondary access can cause a self-deadlock when reading via a
  * secondary because the collections API secondary implementation in DB 4.2
@@ -36,9 +38,7 @@ public class SecondaryDeadlockTest extends TestCase {
     private static final int N_ITERS = 20;
     private static final int MAX_RETRIES = 1000;
 
-    public static void main(String[] args)
-        throws Exception {
-
+    public static void main(String[] args) {
         junit.framework.TestResult tr =
             junit.textui.TestRunner.run(suite());
         if (tr.errorCount() > 0 ||
@@ -49,9 +49,7 @@ public class SecondaryDeadlockTest extends TestCase {
         }
     }
 
-    public static Test suite()
-        throws Exception {
-
+    public static Test suite() {
         TestSuite suite = new TestSuite(SecondaryDeadlockTest.class);
         return suite;
     }
@@ -68,6 +66,7 @@ public class SecondaryDeadlockTest extends TestCase {
         super(name);
     }
 
+    @Override
     public void setUp()
         throws Exception {
 
@@ -84,6 +83,7 @@ public class SecondaryDeadlockTest extends TestCase {
                                        true);
     }
 
+    @Override
     public void tearDown() {
 
         if (index != null) {
@@ -139,12 +139,12 @@ public class SecondaryDeadlockTest extends TestCase {
                     /* The TransactionRunner performs retries. */
                     for (int i = 0; i < N_ITERS; i +=1 ) {
                         runner.run(new TransactionWorker() {
-                            public void doWork() throws Exception {
+                            public void doWork() {
                                 assertEquals(null, storeMap.put(N_ONE, N_101));
                             }
                         });
                         runner.run(new TransactionWorker() {
-                            public void doWork() throws Exception {
+                            public void doWork() {
                                 assertEquals(N_101, storeMap.remove(N_ONE));
                             }
                         });
