@@ -1,8 +1,8 @@
 # See the file LICENSE for redistribution information.
 #
-# Copyright (c) 2006,2007 Oracle.  All rights reserved.
+# Copyright (c) 2006-2009 Oracle.  All rights reserved.
 #
-# $Id: test121.tcl,v 1.9 2007/05/24 18:20:47 carol Exp $
+# $Id$
 #
 # TEST	test121
 # TEST	Tests of multi-version concurrency control.
@@ -38,6 +38,8 @@ proc test121 { method {tnum "121"} args } {
 	set encargs ""
 	set args [split_encargs $args encargs]
 	set filename "test.db"
+	set pageargs ""
+	set args [split_pageargs $args pageargs]
 
 	# Create transactional env.  Specifying -multiversion makes
 	# all databases opened within the env -multiversion.
@@ -47,18 +49,18 @@ proc test121 { method {tnum "121"} args } {
 
 	# Raise cachesize so this test focuses on cursor adjustment
 	# and not on small cache issues.
-	set cachesize [expr 1024 * 1024]
+	set cachesize [expr 2 * 1024 * 1024]
 	set max_locks 2000
 	set max_objects 2000
 	set env [eval {berkdb_env -create -cachesize "0 $cachesize 1"}\
 	    -lock_max_locks $max_locks -lock_max_objects $max_objects\
-	    -txn -multiversion $encargs -home $testdir]
+	    -txn -multiversion $encargs $pageargs -home $testdir]
 	error_check_good env_open [is_valid_env $env] TRUE
 
 	# Open database.
 	puts "\tTest$tnum.b: Creating -multiversion db."
 	set db [eval {berkdb_open} \
-	    -create -auto_commit -env $env $omethod $args $filename]
+	    -create -auto_commit -env $env $omethod $args $pageargs $filename]
 	error_check_good db_open [is_valid_db $db] TRUE
 
 	# Start transactions.
